@@ -44,55 +44,6 @@
 
 #define LENGTH 512
 
-/* get rid of any allocated afp_option buffers. */
-void afp_options_free(struct afp_options *opt)
-{
-	if (opt->hostname)
-        free(opt->hostname);
-	if (opt->adminauthuser)
-        free(opt->adminauthuser);
-	if (opt->configfile)
-        free(opt->configfile);
-    if (opt->fqdn)
-        free(opt->fqdn);
-    if (opt->guest)
-        free(opt->guest);
-    if (opt->listen)
-        free(opt->listen);
-    if (opt->k5realm)
-        free(opt->k5realm);
-    if (opt->k5keytab)
-        free(opt->k5keytab);
-    if (opt->k5service)
-        free(opt->k5service);
-    if (opt->logconfig)
-        free(opt->logconfig);
-    if (opt->logfile)
-        free(opt->logfile);
-    if (opt->loginmesg)
-        free(opt->loginmesg);
-    if (opt->maccodepage)
-        free(opt->maccodepage);
-	if (opt->mimicmodel)
-        free(opt->mimicmodel);
-    if (opt->ntdomain)
-        free(opt->ntdomain);
-    if (opt->ntseparator)
-        free(opt->ntseparator);
-    if (opt->passwdfile)
-        free(opt->passwdfile);
-    if (opt->port)
-        free(opt->port);
-    if (opt->signatureopt)
-        free(opt->signatureopt);
-    if (opt->uamlist)
-        free(opt->uamlist);
-    if (opt->uampath)
-        free(opt->uampath);
-    if (opt->unixcodepage)
-        free(opt->unixcodepage);
-}
-
 /*
  * Show version information about afpd.
  * Used by "afp -v".
@@ -142,6 +93,9 @@ static void show_version( void )
 #endif
 #ifdef CNID_BACKEND_TDB
 	printf( "tdb " );
+#endif
+#ifdef CNID_BACKEND_MYSQL
+	printf( "mysql " );
 #endif
 	puts( "" );
 }
@@ -214,6 +168,27 @@ static void show_version_extended(void )
 #else
 	puts( "No" );
 #endif
+
+	printf( "         D-Bus support:\t" );
+#ifdef HAVE_DBUS_GLIB
+	puts( "Yes" );
+#else
+	puts( "No" );
+#endif
+
+	printf( "     Spotlight support:\t" );
+#ifdef HAVE_TRACKER
+	puts( "Yes" );
+#else
+	puts( "No" );
+#endif
+
+	printf( "         DTrace probes:\t" );
+#ifdef WITH_DTRACE
+	puts( "Yes" );
+#else
+	puts( "No" );
+#endif
 }
 
 /*
@@ -222,6 +197,7 @@ static void show_version_extended(void )
 static void show_paths( void )
 {
 	printf( "              afp.conf:\t%s\n", _PATH_CONFDIR "afp.conf");
+	printf( "           extmap.conf:\t%s\n", _PATH_CONFDIR "extmap.conf");
 	printf( "       state directory:\t%s\n", _PATH_STATEDIR);
 	printf( "    afp_signature.conf:\t%s\n", _PATH_STATEDIR "afp_signature.conf");
 	printf( "      afp_voluuid.conf:\t%s\n", _PATH_STATEDIR "afp_voluuid.conf");
