@@ -1,5 +1,4 @@
 /*
- * $Id: cnid_cdb_resolve.c,v 1.1.4.2 2003/10/21 16:23:54 didg Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -10,7 +9,7 @@
 #include "cnid_cdb_private.h"
 
 /* Return the did/name pair corresponding to a CNID. */
-char *cnid_cdb_resolve(struct _cnid_db *cdb, cnid_t *id, void *buffer, u_int32_t len) {
+char *cnid_cdb_resolve(struct _cnid_db *cdb, cnid_t *id, void *buffer, size_t len) {
     CNID_private *db;
     DBT key, data;
     int rc;
@@ -41,7 +40,7 @@ char *cnid_cdb_resolve(struct _cnid_db *cdb, cnid_t *id, void *buffer, u_int32_t
 
     memcpy(id, (char *)data.data +CNID_DID_OFS, sizeof(cnid_t));
 #ifdef DEBUG
-    LOG(log_info, logtype_default, "cnid_resolve: Returning id = %u, did/name = %s",
+    LOG(log_debug9, logtype_default, "cnid_resolve: Returning id = %u, did/name = %s",
         ntohl(*id), (char *)data.data + CNID_NAME_OFS);
 #endif
     return (char *)data.data +  CNID_NAME_OFS;
