@@ -1,5 +1,4 @@
 /*
- * $Id: cnid_tdb_close.c,v 1.1.2.1 2003/09/09 16:42:21 didg Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -17,8 +16,6 @@ void cnid_tdb_close(struct _cnid_db *cdb)
     free(cdb->volpath);
     db = (struct _cnid_tdb_private *)cdb->_private;
     tdb_close(db->tdb_cnid);
-    tdb_close(db->tdb_didname);
-    tdb_close(db->tdb_devino);
     free(cdb->_private);
     free(cdb);
 }
