@@ -86,6 +86,8 @@ int sys_get_easize(VFS_FUNC_ARGS_EA_GETSIZE)
 
         case ENOATTR:
         case ENOENT:
+            if (vol->v_obj->afp_version >= 34)
+                return AFPERR_NOITEM;
             return AFPERR_MISC;
 
         default:
@@ -161,6 +163,8 @@ int sys_get_eacontent(VFS_FUNC_ARGS_EA_GETCONTENT)
             return AFPERR_MISC;
 
         case ENOATTR:
+            if (vol->v_obj->afp_version >= 34)
+                return AFPERR_NOITEM;
             return AFPERR_MISC;
 
         default:
@@ -313,8 +317,13 @@ int sys_set_ea(VFS_FUNC_ARGS_EA_SET)
             LOG(log_debug, logtype_afpd, "sys_set_ea(\"%s/%s\", ea:'%s'): EA already exists",
                 getcwdpath(), uname, attruname);
             return AFPERR_EXIST;
+        case ENOATTR:
+        case ENOENT:
+            if ((attr_flag & XATTR_REPLACE) && (vol->v_obj->afp_version >= 34))
+                return AFPERR_NOITEM;
+            return AFPERR_MISC;
         default:
-            LOG(log_error, logtype_afpd, "sys_set_ea(\"%s/%s\", ea:'%s', size: %u, flags: %s|%s|%s): %s",
+            LOG(log_debug, logtype_afpd, "sys_set_ea(\"%s/%s\", ea:'%s', size: %u, flags: %s|%s|%s): %s",
                 getcwdpath(), uname, attruname, attrsize, 
                 oflag & O_CREAT ? "XATTR_CREATE" : "-",
                 oflag & O_TRUNC ? "XATTR_REPLACE" : "-",
@@ -439,6 +448,9 @@ int sys_ea_copyfile(VFS_FUNC_ARGS_COPYFILE)
 		if (!*name)
 			continue;
 
+        if (STRCMP(name, ==, AD_EA_META))
+            continue;
+
         if (sfd != -1) {
             if (fchdir(sfd) == -1) {
                 LOG(log_error, logtype_afpd, "sys_ea_copyfile: cant chdir to sfd: %s",
